/*
  Copyright (C) 2001-2003 Mahlon E. Smith <mahlon@martini.nu>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either
  version 2 of the License, or any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <gtk/gtk.h>
#include <errno.h>
#include <unistd.h>
#include "globals.h"
#include "gtkglobals.h"
#include "../imgs/shock.xpm"

int main(
	 int argc,
	 char *argv[]
	 ) 
{

  /* Define variables */
  int fpid = 0; /* fork return val */

  /* Define functions */
  gint delete_event(), update();
  void destroy(), get_main_menu(), gerror();
  int getprefs(), getip(), get_gwip();

  /* Define widgets for mainwindow */
  GtkWidget *mainwindow, *imagebox, *actionbutton, *label;
  GtkWidget *layoutvbox1, *layoutvbox2;
  GtkWidget *menubar;
  GtkTooltips *tooltips;
  GtkWidget *shock;
  GdkPixmap *shockimage;
  GdkBitmap *shockmask;
  GtkStyle *shockstyle;
  
  /* Fork away from the command line if not in debug mode. */
#ifndef DEBUG
  fpid = fork();
#endif DEBUG
  
  if (fpid != 0) {  
    /* We're the parent, die */
    exit(0); 
  } 
  if (fpid == -1) { 
    /* something went wrong */
    g_print("Unable to fork: %d\n", errno);
    exit(1);
  }

  /* Fire up Gtk routines */
  gtk_init(&argc, &argv);

  /* Create mainwindow */
  mainwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  /* Set title */
  gtk_window_set_title(GTK_WINDOW(mainwindow), PACKAGE_STRING);
  /* Sets the border width of the window. */
  gtk_container_set_border_width(GTK_CONTAINER(mainwindow), 0);
  /* Set the default window size */
  /* gtk_window_set_default_size(GTK_WINDOW(mainwindow), 200, 350) */;
  gtk_widget_set_usize(GTK_WIDGET(mainwindow), 200, 335);
  /* Disallow resizing - modal mode */
  gtk_window_set_policy(GTK_WINDOW(mainwindow), FALSE, FALSE, FALSE);
  /* Link destroy signals to mainwindow */
  gtk_signal_connect(GTK_OBJECT(mainwindow), "delete_event",
		     GTK_SIGNAL_FUNC(delete_event), NULL);
  gtk_signal_connect(GTK_OBJECT(mainwindow), "destroy",
		     GTK_SIGNAL_FUNC(destroy), NULL);
  
  /* Create and pack layout frame 1 */
  /* gtk_vbox_new(samesize?, spacing); */
  layoutvbox1 = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(mainwindow), layoutvbox1);

  /* Generate the menubar */
  get_main_menu(mainwindow, &menubar);
  gtk_box_pack_start(GTK_BOX(layoutvbox1), menubar, FALSE, TRUE, 0);

  /* Create and pack layout frame 2 */
  layoutvbox2 = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(layoutvbox1), layoutvbox2);
  /* Give everything under the menubar some padding */
  gtk_container_set_border_width(GTK_CONTAINER(layoutvbox2), 10);

  /* Create and place the image box */
  imagebox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(layoutvbox2), imagebox, FALSE, FALSE, 5);
  /* Setup and place image */
  shockstyle = gtk_widget_get_style(mainwindow);
  /* If window not shown, it needs to be realized for the image to draw */
  gtk_widget_realize(mainwindow);
  shockimage = gdk_pixmap_create_from_xpm_d(mainwindow->window, &shockmask,
					    &shockstyle->bg[GTK_STATE_NORMAL],
					    (gchar **)shock_xpm);
  shock = gtk_pixmap_new(shockimage, shockmask);
  gtk_container_add(GTK_CONTAINER(imagebox), shock);
  
  /* A little label padding... */
  label = gtk_label_new("   ");
  gtk_container_add(GTK_CONTAINER(layoutvbox2), label);

  /* Hostname/Login label */
  label = gtk_label_new("hostname:");
  gtk_container_add(GTK_CONTAINER(layoutvbox2), label);
  gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.0f);
  /* Hostname/Login textbox */
  hostnamebox = gtk_entry_new_with_max_length(50);
  gtk_container_add(GTK_CONTAINER(layoutvbox2), hostnamebox);

  /* A little label padding... */
  label = gtk_label_new("   ");
  gtk_container_add(GTK_CONTAINER(layoutvbox2), label);

  /* Password label */
  label = gtk_label_new("password:");
  gtk_container_add(GTK_CONTAINER(layoutvbox2), label);
  gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.0f);
  /* Password field */
  passwordbox = gtk_entry_new_with_max_length(100);
  gtk_container_add(GTK_CONTAINER(layoutvbox2), passwordbox);
#ifndef DEBUG
  gtk_entry_set_visibility(GTK_ENTRY(passwordbox), FALSE);
#endif DEBUG

  /* A little label padding... */
  label = gtk_label_new("   ");
  gtk_container_add(GTK_CONTAINER(layoutvbox2), label);

  /* IP address label */
  label = gtk_label_new("ip address:");
  gtk_container_add(GTK_CONTAINER(layoutvbox2), label);
  gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.0f);
  /* IP address textbox */
  ipbox = gtk_entry_new_with_max_length(15);
  gtk_container_add(GTK_CONTAINER(layoutvbox2), ipbox);

  /* A little label padding... */
  label = gtk_label_new("   ");
  gtk_container_add(GTK_CONTAINER(layoutvbox2), label);

  /* Status line */
  status_bar = gtk_statusbar_new();
  gtk_box_pack_end(GTK_BOX(layoutvbox2), status_bar, FALSE, FALSE, 2);
  status_id = gtk_statusbar_get_context_id(
	      GTK_STATUSBAR(status_bar), "statusbar");

  /* Create and place action button */
  actionbutton = gtk_button_new_with_label("update");
  /*  gtk_box_pack_start(GTK_BOX(boxname), widget, expand?, fill?, padding); */
  gtk_box_pack_end(GTK_BOX(layoutvbox2), actionbutton, FALSE, FALSE, 2);
  /* Link signals to button */
  gtk_signal_connect(GTK_OBJECT(actionbutton), "clicked",
		     GTK_SIGNAL_FUNC(update), NULL);

  /* Set tooltips */
  tooltips = gtk_tooltips_new();
  gtk_tooltips_set_tip(tooltips, actionbutton, "Connect and update IP", NULL);

  /* Populate fields with previously saved info, if prefs exist */
  if (!getprefs())
    {
      /* Populate username/password */
      gtk_entry_set_text(GTK_ENTRY(hostnamebox), hostname);
      gtk_entry_set_text(GTK_ENTRY(passwordbox), password);

      /* Only populate ip from prefs if curip and getip are disabled. */
      if ((curip == 0) && (gateip == 0)) {
        gtk_entry_set_text(GTK_ENTRY(ipbox), ip);
      } else {
        /* Get the current IP, put it in place, and disable the field. */
         if (curip == 1) {
                if (!getip()) {
                        gtk_entry_set_text(GTK_ENTRY(ipbox), ip);
                        gtk_widget_set_sensitive(ipbox, FALSE);
                } else {
                        gerror("Unable to determine machine IP address.", 0);
                }
          }
          if (gateip == 1) {
                if (!get_gwip()) {
                        gtk_entry_set_text(GTK_ENTRY(ipbox), ip);
                        gtk_widget_set_sensitive(ipbox, FALSE);
                } else {
                        gerror("Unable to determine gateway IP address.", 0);
                }
          }
      }
  }

  /* Display it all - everything linked under the main window. */
  gtk_widget_show_all(mainwindow);

  /* Autoconnect if requested in prefs. */
  if (startc == 1) { update(); }

  /* Enter Gtk loop */
  gtk_main();

  /* This is only called after gtk_main_quit() */
  return(0);
}
