/*
  Copyright (C) 2001-2003 Mahlon E. Smith <mahlon@martini.nu>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either
  version 2 of the License, or any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


/*
  Generate menu layout, as taken from the Gtk tutorials.
*/

#include <gtk/gtk.h>
#include <strings.h>

void get_main_menu(GtkWidget  *window,
		   GtkWidget **menubar)
{
  
  void gerror(), destroy(), about(), showprefs();

  /* This is the GtkItemFactoryEntry structure used to generate new menus.
     Item 1: The menu path. The letter after the underscore indicates an
     accelerator key once the menu is open.
     Item 2: The accelerator key for the entry
     Item 3: The callback function.
     Item 4: The callback action.  This changes the parameters with
     which the function is called.  The default is 0.
     Item 5: The item type, used to define what kind of an item it is.
     Here are the possible values:
     
     NULL               -> "<Item>"
     ""                 -> "<Item>"
     "<Title>"          -> create a title item
     "<Item>"           -> create a simple item
     "<CheckItem>"      -> create a check item
     "<ToggleItem>"     -> create a toggle item
     "<RadioItem>"      -> create a radio item
     <path>             -> path of a radio item to link against
     "<Separator>"      -> create a separator
     "<Branch>"         -> create an item to hold sub items (optional)
     "<LastBranch>"     -> create a right justified branch 
  */
  
  /* Menubar struct */
  static GtkItemFactoryEntry menu_items[] = {
    { "/_File", NULL, NULL, 0, "<Branch>" },
    { "/File/_Preferences", "<control>P", showprefs, 0, NULL },
    { "/File/About", NULL, about, 0, NULL },
    { "/File/sep", NULL, NULL, 0, "<Separator>" },
    { "/File/_Quit",     "<control>Q", destroy, 0, NULL },
  };

  GtkItemFactory *item_factory;
  GtkAccelGroup *accel_group;
  gint nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);
  
  accel_group = gtk_accel_group_new ();
  
  /* This function initializes the item factory.
     Param 1: The type of menu - can be GTK_TYPE_MENU_BAR, GTK_TYPE_MENU,
              or GTK_TYPE_OPTION_MENU.
     Param 2: The path of the menu.
     Param 3: A pointer to a gtk_accel_group.  The item factory sets up
              the accelerator table while generating menus.
  */
  
  item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", 
				      accel_group);
  
  /* This function generates the menu items. Pass the item factory,
     the number of items in the array, the array itself, and any
     callback data for the the menu items. */
  gtk_item_factory_create_items(item_factory, nmenu_items, menu_items, NULL);

  /* Attach the new accelerator group to the window. */
  gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
  
  if (menubar)
    /* Finally, return the actual menu bar created by the item factory. */ 
    *menubar = gtk_item_factory_get_widget (item_factory, "<main>");
}
