/*
  Copyright (C) 2001-2003 Mahlon E. Smith <mahlon@martini.nu>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either
  version 2 of the License, or any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <gtk/gtk.h>
#include "globals.h"
#include "gtkglobals.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <time.h>

/* 
   Check for a preferences file.
   Read in the values, set global vars 
   Returns 0 on success, 1 on failure.
*/
int getprefs()
{
  
  FILE *chargerc;
  char rcfile[200];
  char line[150];
  int count = 0;
  
  /* Initialize vars to off, except for version check */
  curip = pers = startc = 0;
  checkv = 1;

  /* Build path to the prefs file */
  sprintf(rcfile, "%s/.chargerc", getenv("HOME"));

  /* Attempt to open the prefs file. */
  chargerc = fopen(rcfile, "r");
  
  /* No rcfile, startin up from scratch */
  if (chargerc == NULL) { 
#ifdef DEBUG
    printf("No rc file found.\n");
#endif DEBUG
    return(1);
  }

  while(fgets(line, sizeof(line), chargerc) != NULL) {
    /* skip comments */
    if(line[0] == '#') { continue; } 
    /* kill newline */
    if (line[strlen(line) - 1] == '\n') { line[strlen(line) - 1] = '\0'; } 
    count++;

    /* 
       Since we already know what is on each line, 
       we should be ok defining variables via trusting what
       line number they are on. 
    */
    switch (count) {

    case 1:
      strcpy(hostname, line);
#ifdef DEBUG
      printf("%d: hostname -> %s\n", count, line);
#endif DEBUG
      break;

    case 2:
      strcpy(password, line);
#ifdef DEBUG
      printf("%d: password -> %s\n", count, line);
#endif DEBUG
      break;

    case 3:
      strcpy(ip, line);
#ifdef DEBUG
      printf("%d: ip -> %s\n", count, line);
#endif DEBUG
      break;
      
    case 4:
      sscanf(line, "%d", &curip);
#ifdef DEBUG
      printf("%d: curip -> %d\n", count, curip);
#endif DEBUG
      break;
      
    case 5:
      sscanf(line, "%d", &pers);
#ifdef DEBUG
      printf("%d: pers -> %d\n", count, pers);
#endif DEBUG
      break;

    case 6:
      sscanf(line, "%d", &startc);
#ifdef DEBUG
      printf("%d: startc -> %d\n", count, startc);
#endif DEBUG
      break;

    case 7:
      sscanf(line, "%d", &checkv);
#ifdef DEBUG
      printf("%d: checkv -> %d\n", count, checkv);
#endif DEBUG
      break;

    case 8:
      sscanf(line, "%d", &gateip);
#ifdef DEBUG
      printf("%d: gateip -> %d\n", count, gateip);
#endif DEBUG
      break;

    case 9:
      sscanf(line, "%d", &uproxy);
#ifdef DEBUG
      printf("%d: uproxy -> %d\n", count, uproxy);
#endif DEBUG
      break;

    case 10:
      strcpy(proxy_server, line);
#ifdef DEBUG
      printf("%d: proxy -> %s\n", count, line);
#endif DEBUG
      break;

    default:
      /* Ignore anything not in the 1st 10 lines */
      break;

    }
  }
  fclose(chargerc);
  return(0);
}

/*------------------------------------------------------------*/

/* Read all set global vars, 
   write it out to a prefs file */
int saveprefs()
{

  FILE *chargerc;
  char rcfile[200];
  time_t seconds;
  char *date;

  void update_status();

  /* Build path to file */
  sprintf(rcfile, "%s/.chargerc", getenv("HOME"));

#ifdef DEBUG
  printf("Saving settings to: %s\n", rcfile);
#endif DEBUG

  chargerc = fopen(rcfile, "w");

  time(&seconds);
  date = ctime(&seconds);
  fputs("# StaticCharge prefs file created ", chargerc);
  fputs(date, chargerc);
  fputs("# This file is automatically generated - do not edit!\n", chargerc);

  /* Grab the currently entered info in the textfields,
	stick it into the global vars */
  strcpy(hostname, gtk_entry_get_text(GTK_ENTRY(hostnamebox)));
  strcpy(password, gtk_entry_get_text(GTK_ENTRY(passwordbox)));
  strcpy(ip, gtk_entry_get_text(GTK_ENTRY(ipbox)));

  /* Copying from a textfield, from a window that isn't active doesn't seem to work. */
  if (prefwin == 1) {
    strcpy(proxy_server, gtk_entry_get_text(GTK_ENTRY(proxy_serverbox)));
  }

  fputs(hostname, chargerc);
  fputs("\n", chargerc);

  if (pers == 1) {
    fputs(password, chargerc);
  }
  fputs("\n", chargerc);

  fputs(ip, chargerc);
  fputs("\n", chargerc);

  (curip == 1) ? fputs("1", chargerc) : fputs("0", chargerc);
  fputs("\n", chargerc);

  (pers == 1) ? fputs("1", chargerc) : fputs("0", chargerc);
  fputs("\n", chargerc);

  (startc == 1) ? fputs("1", chargerc) : fputs("0", chargerc);
  fputs("\n", chargerc);

  (checkv == 1) ? fputs("1", chargerc) : fputs("0", chargerc);
  fputs("\n", chargerc);

  (gateip == 1) ? fputs("1", chargerc) : fputs("0", chargerc);
  fputs("\n", chargerc);

  (uproxy == 1) ? fputs("1", chargerc) : fputs("0", chargerc);
  fputs("\n", chargerc);

  fputs(proxy_server, chargerc);
  fputs("\n", chargerc);

  chmod(rcfile, 0600);

  fclose(chargerc);

  if (prefwin == 1) {
    update_status("Preferences saved.");
  }

  prefwin = 0;
  return(0);
}
