/*
  Copyright (C) 2001-2003 Mahlon E. Smith <mahlon@martini.nu>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either
  version 2 of the License, or any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


/*
  window.c
  Functions relating to creation of windows - 
  error, about, preferences, etc
*/

#include <gtk/gtk.h>
#include "globals.h"
#include "gtkglobals.h"
#include "../imgs/max.xpm"
#include <stdio.h>
#include <string.h>

/* ------------------------------------------------------------------------------- */

/* 
   Display preferences window.
*/
void showprefs()
{

  void closewin(), curip_toggle(), gateway_ip_toggle(), savepass_toggle(), 
		startc_toggle(), checkv_toggle(), proxy_toggle();
  void reset_prefs();
  gint delete_event();
  int saveprefs();

  /* Create widgets */
  GtkWidget *prefwindow, *cancelbutton, *savebutton, *label;
  GtkWidget *layoutvbox, *layoutvbox_general, *buttonbox, *selectionbox;
  GtkWidget *proxybox, *use_proxy, *layoutvbox_proxy;
  GtkWidget *use_curip, *gateway_ip, *save_pass, *connect_start, *check_version;
  GtkTooltips *tooltips;
  GtkWidget *notebook;
  
  /* Set var for pref saving */
  prefwin = 1;

  /* Create window */
  prefwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(prefwindow), "preferences");
  gtk_window_set_modal(GTK_WINDOW(prefwindow), TRUE);
  gtk_widget_set_usize(GTK_WIDGET(prefwindow), 300, 0);
  gtk_container_set_border_width(GTK_CONTAINER(prefwindow), 10);
  gtk_window_set_position(GTK_WINDOW(prefwindow), GTK_WIN_POS_CENTER);
  gtk_window_set_policy(GTK_WINDOW(prefwindow), FALSE, FALSE, FALSE);
  gtk_signal_connect(GTK_OBJECT(prefwindow), "delete_event",
		     GTK_SIGNAL_FUNC(delete_event), NULL);

  /* Create main layout box */
  layoutvbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(prefwindow), layoutvbox);
  gtk_widget_show(layoutvbox);

  /* Create notebook */
  notebook = gtk_notebook_new();
  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
  gtk_container_add(GTK_CONTAINER(layoutvbox), notebook);
  gtk_notebook_set_show_border(GTK_NOTEBOOK(notebook), 0);
  gtk_widget_show(notebook);

  /* Create layout boxes for within the notebook */
  layoutvbox_general = gtk_vbox_new(FALSE, 0);
  layoutvbox_proxy = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(layoutvbox_general);
  gtk_widget_show(layoutvbox_proxy);

  /* Create and place checkboxes */
  use_curip = gtk_check_button_new_with_label(" use current ip address");
  gateway_ip = gtk_check_button_new_with_label(" use gateway ip address ");
  save_pass = gtk_check_button_new_with_label(" save password ");
  connect_start = gtk_check_button_new_with_label(" connect on startup ");
  check_version = gtk_check_button_new_with_label(" check for newer clients ");
  gtk_widget_show(use_curip);
  gtk_widget_show(gateway_ip);
  gtk_widget_show(save_pass);
  gtk_widget_show(connect_start);
  gtk_widget_show(check_version);
  selectionbox = gtk_vbox_new(FALSE, 4);
  gtk_widget_show(selectionbox);
  gtk_container_add(GTK_CONTAINER(layoutvbox_general), selectionbox);
  /* A little label padding... */
  label = gtk_label_new("   "); 
  gtk_box_pack_start(GTK_BOX(selectionbox), label, FALSE, FALSE, 1);
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(selectionbox), use_curip, FALSE, FALSE, 1);
  gtk_box_pack_start(GTK_BOX(selectionbox), gateway_ip, FALSE, FALSE, 1);
  gtk_box_pack_start(GTK_BOX(selectionbox), save_pass, FALSE, FALSE, 1);
  gtk_box_pack_start(GTK_BOX(selectionbox), connect_start, FALSE, FALSE, 1);
  gtk_box_pack_start(GTK_BOX(selectionbox), check_version, FALSE, FALSE, 1);
  label = gtk_label_new("   ");
  gtk_box_pack_start(GTK_BOX(selectionbox), label, FALSE, FALSE, 1);
  gtk_widget_show(label);
  gtk_signal_connect(GTK_OBJECT(use_curip), "clicked",
		     GTK_SIGNAL_FUNC(curip_toggle),
		     GTK_WIDGET(gateway_ip));
  gtk_signal_connect(GTK_OBJECT(gateway_ip), "clicked",
		     GTK_SIGNAL_FUNC(gateway_ip_toggle),
		     GTK_WIDGET(use_curip));
  gtk_signal_connect(GTK_OBJECT(save_pass), "clicked",
		     GTK_SIGNAL_FUNC(savepass_toggle),
		     GTK_WIDGET(connect_start));
  gtk_signal_connect(GTK_OBJECT(connect_start), "clicked",
		     GTK_SIGNAL_FUNC(startc_toggle),
		     NULL);
  gtk_signal_connect(GTK_OBJECT(check_version), "clicked",
		     GTK_SIGNAL_FUNC(checkv_toggle),
		     NULL);

  /* Create and place proxy widgets */
  label = gtk_label_new("\n\n"); 
  gtk_box_pack_start(GTK_BOX(layoutvbox_proxy), label, FALSE, FALSE, 1);
  gtk_widget_show(label);
  use_proxy = gtk_check_button_new_with_label(" use a proxy server ");
  gtk_box_pack_start(GTK_BOX(layoutvbox_proxy), use_proxy, FALSE, FALSE, 1);
  gtk_widget_show(use_proxy);
  proxybox = gtk_hbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(layoutvbox_proxy), proxybox);
  gtk_widget_show(proxybox);
  label = gtk_label_new("proxy:");
  gtk_box_pack_start(GTK_BOX(proxybox), label, FALSE, TRUE, 10);
  gtk_widget_show(label);
  proxy_serverbox = gtk_entry_new_with_max_length(50);
  gtk_box_pack_start(GTK_BOX(proxybox), proxy_serverbox, TRUE, TRUE, 10);
  gtk_widget_show(proxy_serverbox);
  label = gtk_label_new("example:  proxy.yourisp.net:3128");
  gtk_box_pack_start(GTK_BOX(layoutvbox_proxy), label, FALSE, TRUE, 10);
  gtk_widget_show(label);
  label = gtk_label_new("\n\n"); 
  gtk_box_pack_start(GTK_BOX(layoutvbox_proxy), label, FALSE, FALSE, 1);
  gtk_widget_show(label);
  gtk_signal_connect(GTK_OBJECT(use_proxy), "clicked",
		     GTK_SIGNAL_FUNC(proxy_toggle),
		     GTK_WIDGET(proxy_serverbox)); 

  /* Some padding in between the notebook and the control buttons */
  label = gtk_label_new("   "); 
  gtk_container_add(GTK_CONTAINER(layoutvbox), label);
  gtk_widget_show(label);

  /* Cancel/Save layout box */
  buttonbox = gtk_hbox_new(FALSE, 4);
  gtk_container_add(GTK_CONTAINER(layoutvbox), buttonbox);
  gtk_widget_show(buttonbox);

  /* Place buttons */
  savebutton = gtk_button_new_with_label("   save prefs   ");
  gtk_widget_show(savebutton);
  gtk_box_pack_end(GTK_BOX(buttonbox), savebutton, FALSE, FALSE, 3);
  cancelbutton = gtk_button_new_with_label("   cancel   ");
  gtk_widget_show(cancelbutton);
  gtk_box_pack_end(GTK_BOX(buttonbox), cancelbutton, FALSE, FALSE, 3);
  gtk_signal_connect(GTK_OBJECT(savebutton), "clicked",
		     GTK_SIGNAL_FUNC(saveprefs),
		     NULL);
  gtk_signal_connect(GTK_OBJECT(savebutton), "clicked",
		     GTK_SIGNAL_FUNC(closewin),
		     GTK_WIDGET(prefwindow));
  gtk_signal_connect(GTK_OBJECT(cancelbutton), "clicked",
		     GTK_SIGNAL_FUNC(reset_prefs),
		     NULL);
  gtk_signal_connect(GTK_OBJECT(cancelbutton), "clicked",
		     GTK_SIGNAL_FUNC(closewin),
		     GTK_WIDGET(prefwindow));

  /* Set tooltips. */
  tooltips = gtk_tooltips_new();
  gtk_tooltips_set_tip(tooltips, use_curip, "Use the IP address currently bound to this machine's outside interface.", NULL);
  gtk_tooltips_set_tip(tooltips, gateway_ip, "If you are behind a NAT router, this selection finds your outside gateway IP address.", NULL);
  gtk_tooltips_set_tip(tooltips, save_pass, "Toggle saving your password into an .rc file in your homedir.", NULL);
  gtk_tooltips_set_tip(tooltips, connect_start, "If you are saving your password, this toggles automatic connecting at application startup.", NULL);
  gtk_tooltips_set_tip(tooltips, check_version, "Toggle notification of available clients that are newer than this one.", NULL);
  gtk_tooltips_set_tip(tooltips, use_proxy, "Connect via an external proxy server.", NULL);
  gtk_tooltips_set_tip(tooltips, proxy_serverbox, "Hostname and port number of proxy server.", NULL);

  /* Hang onto current global var values, so we can revert
     if they hit the cancel button */
  strcpy(ip_tmp, ip);
  curip_tmp = curip;
  gateip_tmp = gateip;
  pers_tmp = pers;
  startc_tmp = startc;
  checkv_tmp = checkv;
  uproxy_tmp = uproxy; 
  strcpy(proxy_tmp, proxy_server);

  /* Preset toggles to appropriate values, based on global vars */
  if (curip == 1) {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_curip), TRUE);
  } else {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_curip), FALSE);    
  }
  if (gateip == 1) {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gateway_ip), TRUE);
  } else {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gateway_ip), FALSE);    
  }
  if (pers == 1) {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(save_pass), TRUE);
    gtk_widget_set_sensitive(connect_start, TRUE);
  } else {
    gtk_widget_set_sensitive(connect_start, FALSE);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(save_pass), FALSE);    
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(connect_start), FALSE);
  }
  if (startc == 1) {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(connect_start), TRUE);
  } else {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(connect_start), FALSE);    
  }
  if (checkv == 1) {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_version), TRUE);
  } else {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_version), FALSE);    
  }
  if (uproxy == 1) {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_proxy), TRUE);
  } else {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_proxy), FALSE);    
    gtk_widget_set_sensitive(proxy_serverbox, FALSE);
  }
  gtk_entry_set_text(GTK_ENTRY(proxy_serverbox), proxy_server);

  /* Create notebook pages */
  label = gtk_label_new("  general  "); 
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), layoutvbox_general, label);
  label = gtk_label_new("  proxy  "); 
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), layoutvbox_proxy, label);

  /* Show window */
  gtk_widget_show(prefwindow);

  return;
}

/* ------------------------------------------------------------------------------- */

/* Reset preferences variables back to previous settings */
void reset_prefs()
{

  void update_status();

  curip = curip_tmp;
  gateip = gateip_tmp;
  pers = pers_tmp;
  startc = startc_tmp;
  checkv = checkv_tmp;
  uproxy = uproxy_tmp;
  strcpy(ip, ip_tmp);
  strcpy(proxy_server, proxy_tmp);
    
  prefwin = 0;
  gtk_entry_set_text(GTK_ENTRY(ipbox), ip_tmp);

  if ((curip == 0 && gateip == 0)) {
    gtk_widget_set_sensitive(ipbox, TRUE);
  } else {
    gtk_widget_set_sensitive(ipbox, FALSE);
  }

  update_status("Preferences cancelled.");

  return;

}

/* ------------------------------------------------------------------------------- */

/* 
   Checkbox callbacks.
   Set global variables, perform enabling/disabling
   of various features.
*/
void curip_toggle (
		   GtkWidget *widget,
		   GtkWidget *gateway_ip
		   )
{

  int getip();
  void gerror();

  if (GTK_TOGGLE_BUTTON(widget)->active) 
    {
      curip = 1;
#ifdef DEBUG
      g_print("Use current IP: active\n");
#endif DEBUG

      gtk_widget_set_sensitive(gateway_ip, FALSE);
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gateway_ip), FALSE);

      if (!getip()) {
	gtk_entry_set_text(GTK_ENTRY(ipbox), ip);
	gtk_widget_set_sensitive(ipbox, FALSE);
      } else {
	gerror("Unable to determine machine IP address.", 0);
      }

    } else {
      curip = 0;
      gtk_widget_set_sensitive(gateway_ip, TRUE);
      gtk_entry_set_text(GTK_ENTRY(ipbox), ip_tmp);
      gtk_widget_set_sensitive(ipbox, TRUE);
#ifdef DEBUG
      g_print("Use current IP: inactive\n");
#endif DEBUG
    }
  return;
}


void gateway_ip_toggle (
		   GtkWidget *widget,
		   GtkWidget *use_curip
		   )
{
  
  int get_gwip();
  void gerror();

  if (GTK_TOGGLE_BUTTON(widget)->active) 
    {
#ifdef DEBUG
      g_print("Use gateway IP: active\n");
#endif DEBUG
      if (gateip == 1) { /* already set, don't get IP again */
	return;
      }
      gateip = 1;
      
      gtk_widget_set_sensitive(use_curip, FALSE);
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_curip), FALSE);

      if (!get_gwip()) {  /* Specific error codes required... */
	gtk_entry_set_text(GTK_ENTRY(ipbox), ip); 
	gtk_widget_set_sensitive(ipbox, FALSE);
      } else {
	gerror("Unable to determine gateway IP address.", 0);
      }

    } else {
      gateip = 0;
      gtk_widget_set_sensitive(use_curip, TRUE);
      gtk_entry_set_text(GTK_ENTRY(ipbox), ip_tmp);
      gtk_widget_set_sensitive(ipbox, TRUE);
#ifdef DEBUG
      g_print("Use gateway IP: inactive\n");
#endif DEBUG
    }
  return;
}

void proxy_toggle (
		   GtkWidget *widget,
		   GtkWidget *proxy_serverbox
		   )
{
  if (GTK_TOGGLE_BUTTON(widget)->active) 
    {
      uproxy = 1;
      gtk_widget_set_sensitive(proxy_serverbox, TRUE);
#ifdef DEBUG
      g_print("Use proxy: active\n");
#endif DEBUG
    } else {
      uproxy = 0;
      gtk_widget_set_sensitive(proxy_serverbox, FALSE);
#ifdef DEBUG
      g_print("Use proxy: inactive\n");
#endif DEBUG
    }
  return;
}


void savepass_toggle (
		      GtkWidget *widget,
		      GtkWidget *connect_start
		      )
{
  if (GTK_TOGGLE_BUTTON(widget)->active) 
    {
      pers = 1;
      gtk_widget_set_sensitive(connect_start, TRUE);
#ifdef DEBUG
      g_print("Save password: active\n");
#endif DEBUG
    } else {
      startc = 0;
      pers = 0;
      gtk_widget_set_sensitive(connect_start, FALSE);
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(connect_start), FALSE);
#ifdef DEBUG
      g_print("Save password: inactive\n");
#endif DEBUG
    }
  return;
}

void startc_toggle (
		   GtkWidget *widget
		   )
{
  if (GTK_TOGGLE_BUTTON(widget)->active) 
    {
      startc = 1;
#ifdef DEBUG
      g_print("Connect at startup: active\n");
#endif DEBUG
    } else {
      startc = 0;
#ifdef DEBUG
      g_print("Connect at startup: inactive\n");
#endif DEBUG
    }
  return;
}

void checkv_toggle (
		   GtkWidget *widget
		   )
{
  if (GTK_TOGGLE_BUTTON(widget)->active) 
    {
      checkv = 1;
#ifdef DEBUG
      g_print("Check for newer versions: active\n");
#endif DEBUG
    } else {
      checkv = 0;
#ifdef DEBUG
      g_print("Check for newer versions: inactive\n");
#endif DEBUG
    }
  return;
}


/* ---------------------------------------------------------------- */


/* 
   Display aboutbox information.
*/
void about()
{

  void closewin();
  gint delete_event();
  int launch_url();
  char message[200];

  /* Create widgets */
  GtkWidget *aboutwindow, *aboutmessage, *aboutbutton;
  GtkWidget *frame, *layoutvbox, *layouthbox1, *layouthbox2, *imagebox;
  GtkWidget *max;
  GdkPixmap *maximage;
  GdkBitmap *maxmask;
  GtkStyle *maxstyle;

  /* Build about message */
  sprintf(message, "%s\nCreated for StaticCling.org\n6.2003", PACKAGE_STRING);
#ifdef DEBUG
  strcat(message, "\n-- DEBUG mode enabled --");
#endif DEBUG

  /* Create window */
  aboutwindow = gtk_window_new(GTK_WINDOW_POPUP);
  gtk_window_set_title(GTK_WINDOW(aboutwindow), "about");
  gtk_window_set_modal(GTK_WINDOW(aboutwindow), TRUE);
  gtk_widget_set_usize(GTK_WIDGET(aboutwindow), 200, 200);
  gtk_window_set_position(GTK_WINDOW(aboutwindow), GTK_WIN_POS_CENTER);
  gtk_window_set_policy(GTK_WINDOW(aboutwindow), FALSE, FALSE, FALSE);
  gtk_signal_connect(GTK_OBJECT(aboutwindow), "delete_event",
		     GTK_SIGNAL_FUNC(delete_event), NULL);

  /* Graphic frame */
  frame = gtk_frame_new(NULL);
  gtk_container_add(GTK_CONTAINER(aboutwindow), frame);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);

  /* Vert layout box */
  layoutvbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(frame), layoutvbox);

  /* Setup and place image */
  layouthbox1 = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(layoutvbox), layouthbox1, FALSE, FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(layouthbox1), 20);
  maxstyle = gtk_widget_get_style(aboutwindow);
  gtk_widget_realize(aboutwindow);
  maximage = gdk_pixmap_create_from_xpm_d(aboutwindow->window, &maxmask,
					  &maxstyle->bg[GTK_STATE_NORMAL],
					  (gchar **)max_xpm);
  max = gtk_pixmap_new(maximage, maxmask);
  imagebox = gtk_event_box_new();
  gtk_box_pack_start(GTK_BOX(layouthbox1), imagebox, TRUE, FALSE, 0);
  gtk_container_add(GTK_CONTAINER(imagebox), max);
  gtk_widget_set_events(imagebox, GDK_BUTTON_PRESS_MASK);
  gtk_signal_connect(GTK_OBJECT(imagebox), "button_press_event",
		     GTK_SIGNAL_FUNC(launch_url), NULL);
  gtk_widget_realize(imagebox);
  gdk_window_set_cursor(imagebox->window, gdk_cursor_new(GDK_HAND2));

  /* Place message */
  aboutmessage = gtk_label_new(message);
  gtk_container_add(GTK_CONTAINER(layoutvbox), aboutmessage);

  /* Horz layout box */
  layouthbox2 = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(layoutvbox), layouthbox2, FALSE, FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(layouthbox2), 20);

  /* Place button */
  aboutbutton = gtk_button_new_with_label("   ok   ");
  gtk_box_pack_start (GTK_BOX(layouthbox2), aboutbutton, TRUE, FALSE, 0);
  gtk_signal_connect(GTK_OBJECT(aboutbutton), "clicked",
		     GTK_SIGNAL_FUNC(closewin),
		     GTK_WIDGET(aboutwindow));
    
  /* Show widgets */
  gtk_widget_show_all(aboutwindow);

  return;

}


/* ---------------------------------------------------------- */


/* 
   Spit out an error message to a Gtk dialogbox.
   Quit or continue, based on severity. 
   gerror("error", 0);
*/
void gerror(
	    char err_message[], 
	    int fatal
	    ) 
{

  void destroy();
  void closewin();
  gint delete_event();
  
  /* Create widgets */
  GtkWidget *errwindow, *errmessage, *errbutton;

  /* Create window using dialog shortcut */
  errwindow = gtk_dialog_new();
  if (fatal == 0) {
    gtk_window_set_title(GTK_WINDOW(errwindow), " ");
  } else {
    gtk_window_set_title(GTK_WINDOW(errwindow), "fatal error");
  }
  gtk_window_set_modal(GTK_WINDOW(errwindow), TRUE);
  gtk_widget_set_usize(GTK_WIDGET(errwindow), 300, 100);
  gtk_window_set_position(GTK_WINDOW(errwindow), GTK_WIN_POS_CENTER);
  gtk_window_set_policy(GTK_WINDOW(errwindow), FALSE, FALSE, FALSE);
  gtk_signal_connect(GTK_OBJECT(errwindow), "delete_event",
		     GTK_SIGNAL_FUNC(delete_event), NULL);

  if (fatal == 1) {
    gtk_signal_connect(GTK_OBJECT(errwindow), "destroy",
		       GTK_SIGNAL_FUNC(destroy), NULL);
  }

  /* Place message */
  errmessage = gtk_label_new(err_message);
  gtk_box_pack_start(GTK_BOX (GTK_DIALOG(errwindow)->vbox),
		     errmessage, TRUE, TRUE, 10);
  
  /* Place button */
  if (fatal == 0) {
    errbutton = gtk_button_new_with_label("   OK   ");
  } else {
    errbutton = gtk_button_new_with_label("   Quit   ");
  }
  gtk_box_pack_start(GTK_BOX (GTK_DIALOG(errwindow)->action_area),
		     errbutton, FALSE, FALSE, 10);

  /* Attach signals to button */
  if (fatal == 0) {
    gtk_signal_connect(GTK_OBJECT(errbutton), "clicked",
		       GTK_SIGNAL_FUNC(closewin),
		       GTK_WIDGET(errwindow));
  } else {
    /* Fatal error - close the app. 
       Perhaps link a logging function here later */
    gtk_signal_connect(GTK_OBJECT(errbutton), "clicked",
		       GTK_SIGNAL_FUNC(destroy), NULL);
  }
  
  /* Show widgets */
  gtk_widget_show(errmessage);
  gtk_widget_show(errbutton);
  gtk_widget_show(errwindow);

  return;

}



